if RequiredScript == "lib/managers/menumanager" then

GraceCounter = GraceCounter or {}
GraceCounter.prev_next_allowed_dmg_t = -100
GraceCounter.was_downed = false
GraceCounter.graces = 0
GraceCounter.grace_downs = 0
GraceCounter.color = Color(255, 255, 220, 0) / 255
GraceCounter.color_downs = Color(255, 255, 150, 0) / 255
GraceCounter.playerdamage = nil
GraceCounter.hudheisttimer = nil

GraceCounter.settings = {
	chat_on_grace = false,
	chat_use_timestamp = true,
	chat_only_lethal_grace = false,
}

function GraceCounter.Reset()
	GraceCounter.prev_next_allowed_dmg_t = -100
	GraceCounter.was_downed = false
	GraceCounter.graces = 0
	GraceCounter.grace_downs = 0
end

function GraceCounter.AddGrace()
	GraceCounter.graces = GraceCounter.graces + 1
end
function GraceCounter.AddGraceDown()
	GraceCounter.grace_downs = GraceCounter.grace_downs + 1
end

function GraceCounter.GetTimeStr()
	local time = GraceCounter.hudheisttimer._last_time or 0 -- TimerManager:game():time()
	-- hudheisttimer text code
	time = math.floor(time)
	local hours = math.floor(time / 3600)
	time = time - hours * 3600
	local minutes = math.floor(time / 60)
	time = time - minutes * 60
	local seconds = math.round(time)
	local text = hours > 0 and (hours < 10 and "0" .. hours or hours) .. ":" or ""
	local text = text .. (minutes < 10 and "0" .. minutes or minutes) .. ":" .. (seconds < 10 and "0" .. seconds or seconds)

	return text
end

-- add settings

local menu_id = "GraceCountMenu"

if io.file_is_readable(SavePath .. "grace_count.json") then
	for k, v in pairs(io.load_as_json(SavePath .. "grace_count.json") or {}) do
		GraceCounter.settings[k] = v
	end
end

Hooks:Add("MenuManagerBuildCustomMenus", "MenuManagerBuildCustomMenus_GraceCount", function(menu_manager, nodes)
	MenuCallbackHandler.GraceCount_toggle = function(self, item)
		GraceCounter.settings[item:name()] = item:value() == "on"
	end

	MenuCallbackHandler.GraceCount_save = function(self)
		io.save_as_json(GraceCounter.settings, SavePath .. "grace_count.json")
	end

	MenuHelper:NewMenu(menu_id)

	MenuHelper:AddToggle({
		id = "chat_on_grace",
		localized = false,
		title = "Chat On Grace",
		desc = "Sends a message in chat when you get graced.",
		callback = "GraceCount_toggle",
		value = GraceCounter.settings.chat_on_grace,
		menu_id = menu_id,
		priority = 3
	})
	MenuHelper:AddToggle({
		id = "chat_use_timestamp",
		localized = false,
		title = "Chat Timestamp",
		desc = "Enables timestamp for mid-heist grace related chat messages.",
		callback = "GraceCount_toggle",
		value = GraceCounter.settings.chat_use_timestamp,
		menu_id = menu_id,
		priority = 2
	})
	MenuHelper:AddToggle({
		id = "chat_only_lethal_grace",
		localized = false,
		title = "Only Chat Grace Downs",
		desc = "Allows messages only when a grace causes a down.",
		callback = "GraceCount_toggle",
		value = GraceCounter.settings.chat_only_lethal_grace,
		menu_id = menu_id,
		priority = 1
	})

	managers.localization:add_localized_strings({
		GraceCount_menu_name = "Grace Count",
		GraceCount_menu_desc = "..."
	})

	nodes[menu_id] = MenuHelper:BuildMenu(menu_id, { back_callback = "GraceCount_save" })
	MenuHelper:AddMenuItem(nodes["blt_options"], menu_id, "GraceCount_menu_name", "GraceCount_menu_desc")
end)

end

-- hooks

if RequiredScript == "lib/states/missionendstate" then

Hooks:PostHook(MissionEndState, "at_enter", "MissionEndState_at_enter_GraceCounter_Post", function(self, ...)
	local time_str = GraceCounter.graces == 1 and " time!" or " times!"
	local grace_str = GraceCounter.grace_downs == 1 and " grace" or " graces"
	managers.chat:_receive_message(managers.chat.GAME, "Grace Count", "You got graced a total of: " .. GraceCounter.graces .. time_str, GraceCounter.color)
	managers.chat:_receive_message(managers.chat.GAME, "Grace Count", "A total of " .. GraceCounter.grace_downs .. grace_str .. " caused a down!", GraceCounter.color_downs)
	GraceCounter.Reset()
end)

elseif RequiredScript == "lib/managers/playermanager" then

Hooks:PostHook(PlayerManager, "init", "PlayerManager_init_GraceCounter_Post", function(self, ...)
	self:register_message(Message.OnPlayerDamage, "GraceCounter_on_hit", function(attack_data)
		-- exceptions
		-- friendly fire
		if attack_data.attacker_unit and GraceCounter.playerdamage:is_friendly_fire(attack_data.attacker_unit) then
			return
		end
		-- only bullets (normal enemies) and fire (yufu wang?)
		if attack_data.variant ~= "bullet" and attack_data.variant ~= "fire" then
			return
		end
		-- ignore all graces while downed, and make sure this hit happened during the grace period
		if (not GraceCounter.was_downed and managers.player:player_timer():time() < GraceCounter.prev_next_allowed_dmg_t) then
			local was_lethal = false
			GraceCounter.AddGrace()
			-- if lethal
			if (GraceCounter.playerdamage:is_downed()) then
				GraceCounter.AddGraceDown()
				was_lethal = true
			end
			-- chat
			if GraceCounter.settings.chat_on_grace then
				local prefix = ""
				if GraceCounter.settings.chat_use_timestamp then
					prefix = GraceCounter.GetTimeStr() .. " | "
				end
				-- actual messages
				if (not GraceCounter.settings.chat_only_lethal_grace) then
					managers.chat:_receive_message(managers.chat.GAME, "Grace Count", prefix .. "Graces: " .. GraceCounter.graces, GraceCounter.color)
				end
				if was_lethal then
					managers.chat:_receive_message(managers.chat.GAME, "Grace Count", prefix .. "Grace downs: " .. GraceCounter.grace_downs, GraceCounter.color_downs)
				end
			end
		end

		local next_allowed_dmg_t = type(GraceCounter.playerdamage._next_allowed_dmg_t) == "number" and GraceCounter.playerdamage._next_allowed_dmg_t or Application:digest_value(GraceCounter.playerdamage._next_allowed_dmg_t, false)
		GraceCounter.prev_next_allowed_dmg_t = next_allowed_dmg_t
		GraceCounter.was_downed = GraceCounter.playerdamage:is_downed()
	end)
end)

elseif RequiredScript == "lib/units/beings/player/playerdamage" then

Hooks:PostHook(PlayerDamage, "init", "PlayerDamage_init_GraceCounter_Post", function(self, ...)
	GraceCounter.playerdamage = self
end)

elseif RequiredScript == "lib/managers/hud/hudheisttimer" then

Hooks:PostHook(HUDHeistTimer, "init", "HUDHeistTimer_init_GraceCounter_Post", function(self, ...)
	GraceCounter.hudheisttimer = self
end)

end
